<?php
namespace MascotCoreSatnet\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Elementor Hello World
 *
 * Elementor widget for hello world.
 *
 * @since 1.0.0
 */
class TM_Elementor_Movies_Info extends Widget_Base {
	public function __construct($data = [], $args = null) {
		parent::__construct($data, $args);
		$direction_suffix = is_rtl() ? '.rtl' : '';
	}

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'tm-ele-movies-info';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return esc_html__( 'Movies Info', 'mascot-core-satnet' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'tm-elementor-widget-icon';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'tm' ];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'mascot-core-hellojs' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {

		$this->start_controls_section(
			'general',
			[
				'label' => esc_html__( 'General', 'mascot-core-satnet' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_control(
			'item_type', [
				'label' => esc_html__( "Choose Item", 'mascot-core-satnet' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'featured_image' => esc_html__( 'Featured Image', 'mascot-core-satnet' ),
					'title' => esc_html__( 'Title', 'mascot-core-satnet' ),
					'director' => esc_html__( 'Director', 'mascot-core-satnet' ),
					'rate' => esc_html__( 'IMDB Rating', 'mascot-core-satnet' ),
					'film_rating' => esc_html__( 'MPA film rating', 'mascot-core-satnet' ),
					'year' => esc_html__( 'Release Year', 'mascot-core-satnet' ),
					'duration' => esc_html__( 'Duration', 'mascot-core-satnet' ),
					'link' => esc_html__( 'External Link', 'mascot-core-satnet' ),
					'categories' => esc_html__( 'Categories', 'mascot-core-satnet' ),
					'comments' => esc_html__( 'Post Comments', 'mascot-core-satnet' ),
				],
				'default' => 'title'
			]
		);
		$this->add_responsive_control(
			'icon_text_alignment',
			[
				'label' => esc_html__( "Text Alignment", 'mascot-core-satnet' ),
				'type' => Controls_Manager::CHOOSE,
				'label_block' => false,
				'options' => mascot_core_text_align_choose(),
				'selectors' => [
					'{{WRAPPER}}' => 'text-align: {{VALUE}};'
				]
			]
		);
		$this->end_controls_section();










		$this->start_controls_section(
			'title_options',
			[
				'label' => esc_html__( 'Title Options', 'mascot-core-satnet' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
				'condition' => [
					'item_type' => array('title')
				]
			]
		);
		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'title_typography',
				'label' => esc_html__( 'Text Typography', 'mascot-core-satnet' ),
				'selector' => '{{WRAPPER}} .title',
			]
		);
		$this->add_control(
			'title_text_color',
			[
				'label' => esc_html__( "Text Color", 'mascot-core-satnet' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}};'
				]
			]
		);
		$this->add_control(
			'title_theme_colored',
			[
				'label' => esc_html__( "Text Theme Colored", 'mascot-core-satnet' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_theme_color_list(),
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: var(--theme-color{{VALUE}});'
				],
			]
		);
		$this->add_responsive_control(
			'title_margin',
			[
				'label' => esc_html__( 'Margin', 'mascot-core-satnet' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em' ],
				'selectors' => [
					'{{WRAPPER}} .title' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
		$this->end_controls_section();










	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
		switch ($settings['item_type']) {
			case 'title':
				satnet_get_title_area_title();
				break;
			case 'director':
				echo satnet_get_rwmb_group( 'satnet_mascot_' . "movies_mb_settings", 'director' );
				break;
			case 'film_rating':
				echo satnet_get_rwmb_group( 'satnet_mascot_' . "movies_mb_settings", 'film_rating' );
				break;
			case 'rate':
				echo satnet_get_rwmb_group( 'satnet_mascot_' . "movies_mb_settings", 'rate' );
				break;
			case 'year':
				echo satnet_get_rwmb_group( 'satnet_mascot_' . "movies_mb_settings", 'year' );
				break;
			case 'duration':
				echo satnet_get_rwmb_group( 'satnet_mascot_' . "movies_mb_settings", 'duration' );
				break;
			case 'featured_image':
				echo get_the_post_thumbnail( get_the_ID(),'full' );
				break;
			case 'link':
				echo satnet_get_rwmb_group( 'satnet_mascot_' . "movies_mb_settings", 'link' );
				break;
			case 'categories':
				$taxonomy 	= 'movie-genre';
				$terms 		= get_the_terms( get_the_ID(), $taxonomy );
				if ( ! empty( $terms ) ) {
					foreach ( $terms as $term ) {
						$post_terms[] = "<a href='". esc_attr( get_term_link( $term->slug, $taxonomy ) ) . "'> " . esc_html( $term->name ) . "</a>";
					}
					echo join( ', ', $post_terms );
				}
				else echo '<i>No categories.</i>';
				break;
			case 'comments':
				if (is_single ()) comments_template ();
				break;
			default:
				break;
		}
	}
}
